﻿-- -----------------------------------------------------
-- Database QuizFaber x Oracle : Update from v.1 to v.2 
-- -----------------------------------------------------
-- Project : QuizFaber
-- Licence : MIT
-- Author  : Luca Galli
-- Email   : info@quizfaber.com
-- -----------------------------------------------------

CREATE TABLE QuizState (
  Code INT NOT NULL,
  Description VARCHAR(30) NULL,
  PRIMARY KEY (Code));

CREATE TABLE WebLog (
  ID INT NOT NULL,
  UserIdentity VARCHAR(255) NULL,
  SeverityLevel INT NULL,
  DateCreated TIMESTAMP NULL,
  Message CLOB NULL,
  PRIMARY KEY (ID))

CREATE SEQUENCE WEBLOG_ID_SEQ;
  
 CREATE OR REPLACE TRIGGER WEBLOG_ID_TRIG 
	BEFORE INSERT ON WebLog 
	FOR EACH ROW
	WHEN (new.ID IS NULL)
	BEGIN
	  SELECT WEBLOG_ID_SEQ.NEXTVAL
	  INTO   :new.ID
	  FROM   dual;
	END;

CREATE TABLE WebSeverityLevel (
  Level INT NOT NULL,
  Description VARCHAR(20) NULL)

DELETE FROM WebSeverityLevel;
INSERT INTO WebSeverityLevel (Level,Description) VALUES (0,'Debug');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (1,'Info');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (2,'Warning');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (3,'Error');

DELETE FROM QuizState;
INSERT INTO QuizState (Code,Description) VALUES (0,'Draft');
INSERT INTO QuizState (Code,Description) VALUES (1,'Completed');
INSERT INTO QuizState (Code,Description) VALUES (2,'Candidate');
INSERT INTO QuizState (Code,Description) VALUES (10,'Published');
INSERT INTO QuizState (Code,Description) VALUES (98,'Archived');
INSERT INTO QuizState (Code,Description) VALUES (99,'Deleted');

ALTER TABLE Person ADD Source VARCHAR(1) NULL;
ALTER TABLE Quizzes ADD State INT DEFAULT 0 NOT NULL;
ALTER TABLE Quizzes ADD Link VARCHAR(255) NULL;
ALTER TABLE Quizzes ADD Properties CLOB NULL;
ALTER TABLE Questions ADD QuestionName VARCHAR(256) NULL;
ALTER TABLE QuizResult ADD FinalPoints FLOAT(12) NULL;
ALTER TABLE QuizResult ADD ReviewPoints FLOAT(12) NULL;
ALTER TABLE QuizResult ADD NotAnsweredQuestsNum INT NULL;
ALTER TABLE QuizResult ADD NotValuatedQuestsNum INT NULL;
ALTER TABLE QuizResultQuestion ADD Points FLOAT(12) NULL;
ALTER TABLE QuizResultReport ADD Charset VARCHAR(255) NULL;
ALTER TABLE QuizResultReport ADD Language VARCHAR(10) NULL;
ALTER TABLE QuizResultReport ADD Origin CLOB NULL;

ALTER TABLE Person ADD CONSTRAINT FK_PersonRole FOREIGN KEY (UserRole) REFERENCES PersonRole(ID); 
ALTER TABLE Quizzes ADD CONSTRAINT FK_QuizState FOREIGN KEY (STATE) REFERENCES QuizState(Code);
ALTER TABLE Categories ADD CONSTRAINT FK_CategoryQuiz FOREIGN KEY (QZ_ID) REFERENCES Quizzes(QZ_ID) ON DELETE CASCADE; 
ALTER TABLE CustomQuestionTypes ADD CONSTRAINT FK_CustomQuestionType_Quiz FOREIGN KEY (QZ_ID) REFERENCES Quizzes(QZ_ID) ON DELETE CASCADE;
ALTER TABLE QuizResultReport ADD CONSTRAINT FK_Report_QuizResult FOREIGN KEY (ID) REFERENCES QuizResult(ID) ON DELETE CASCADE;

ALTER TABLE QuizResult MODIFY UserName VARCHAR(255);
ALTER TABLE QuizResult MODIFY UserLogin VARCHAR(255);
ALTER TABLE QuizResult MODIFY QuizName VARCHAR(255);
ALTER TABLE QuizResult MODIFY QuizTitle VARCHAR(1024);
ALTER TABLE QuizResult ADD ReviewMark FLOAT(12) NULL;
ALTER TABLE QuizResult ADD ReviewDate TIMESTAMP NULL;
ALTER TABLE QuizResult ADD LowestMark INT DEFAULT 0 NOT NULL;
ALTER TABLE QuizResult ADD RoundMark FLOAT(12) DEFAULT 1 NOT NULL;
ALTER TABLE QuizResult ADD IsDuplicated NUMBER(1,0) NULL;
ALTER TABLE QuizResultQuestion ADD Feedback CLOB NULL;
ALTER TABLE QuizResultQuestion ADD IsCancelled NUMBER(1,0) NULL;

-- ALTER TABLE QuizResultAnswer MODIFY AdditionalText CLOB NULL;
ALTER TABLE QuizResultAnswer ADD (tmpAdditionalText  CLOB);
UPDATE QuizResultAnswer SET tmpAdditionalText=AdditionalText;
COMMIT;
ALTER TABLE QuizResultAnswer DROP COLUMN AdditionalText;
ALTER TABLE QuizResultAnswer RENAME COLUMN tmpAdditionalText TO AdditionalText;

-- ALTER TABLE QuizResultReport ADD CONSTRAINT FK_QuizResult FOREIGN KEY (ID) REFERENCES QuizResult(ID);
-- ALTER TABLE QuizResult ADD CONSTRAINT FK_Quiz FOREIGN KEY (QuizID) REFERENCES Quizzes(QZ_ID);

ALTER TABLE Settings ADD ServerName VARCHAR(50) NULL;
ALTER TABLE Settings ADD ServerDescription VARCHAR(255) NULL;
ALTER TABLE Settings ADD ServerVersion VARCHAR(50) NULL;
ALTER TABLE Settings ADD Environment VARCHAR(50) NULL;
ALTER TABLE Settings ADD PortalBaseUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD PhpMyAdminUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD OwnerUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD FooterText VARCHAR(255) NULL;

CREATE UNIQUE INDEX indexUserIdentity ON Person (UserIdentity);

-- password : admin
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Administrator' AS PersonName,'admin' AS UserIdentity,'21232f297a57a5a743894a0e4a801fc3' AS UserPassword,1 AS UserRole,1 AS IsEnabled FROM DUAL) WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'admin');
-- password : anonymous
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Anonymous' AS PersonName, 'anonymous@anonymous.nowhere' AS UserIdentity,'294de3557d9d00b3d2d8a1e6aab028cf' AS UserPassword,2 AS UserRole,1 AS IsEnabled FROM DUAL) WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'anonymous@anonymous.nowhere');

-- QuizSession
DROP TABLE QuizSession;
CREATE TABLE QuizSession (
  PersonID INT NOT NULL,
  SessionID INT NOT NULL,
  DateCreated TIMESTAMP NOT NULL,
  IpLogin VARCHAR(46) NULL,
  SessionData CLOB NULL,
  DateLastUpdate TIMESTAMP NULL,
  QuizID INT NULL,
  IpLogout VARCHAR(46) NULL,
  UserAgent VARCHAR(255) NULL,
  IsRecoverable NUMBER(1,0) NULL,
  PRIMARY KEY (SessionID),
  CONSTRAINT FK_QuizSessionPerson
    FOREIGN KEY (PersonID)
    REFERENCES Person (ID),
  CONSTRAINT FK_QuizSessionQuizResult
    FOREIGN KEY (QuizID)
    REFERENCES QuizResult (ID));


 CREATE SEQUENCE SESSION_ID_SEQ;
  
 CREATE OR REPLACE TRIGGER SESSION_ID_TRIG 
	BEFORE INSERT ON QuizSession 
	FOR EACH ROW
	WHEN (new.SessionID IS NULL)
	BEGIN
	  SELECT SESSION_ID_SEQ.NEXTVAL
	  INTO   :new.SessionID
	  FROM   dual;
	END;

UPDATE Settings SET DbVersion = 2, DateModified=CURRENT_TIMESTAMP, ServerName='Node.js server', ServerDescription='Application for manage quizzes.',ServerVersion='4.1.0',Environment='DEV',PortalBaseUrl='http://localhost:3313/',OwnerUrl='https://www.quizfaber.com',FooterText='QuizFaber';