﻿-- -----------------------------------------------------
-- Database QuizFaber x PostgreSQL : Update from v.1 to v.2 
-- -----------------------------------------------------
-- Project : QuizFaber
-- Licence : MIT
-- Author  : Luca Galli
-- Email   : info@quizfaber.com
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS QuizState (
  Code INTEGER NOT NULL,
  Description VARCHAR(30) NULL,
  PRIMARY KEY (Code));

CREATE TABLE IF NOT EXISTS WebLog (
  ID INTEGER NOT NULL SERIAL,
  UserIdentity VARCHAR(255) NULL,
  SeverityLevel INTEGER NULL,
  DateCreated TIMESTAMP NULL,
  Message TEXT NULL,
  PRIMARY KEY (ID));

CREATE TABLE IF NOT EXISTS WebSeverityLevel (
  Level INTEGER NOT NULL,
  Description VARCHAR(20) NULL);

DELETE FROM WebSeverityLevel;
INSERT INTO WebSeverityLevel (Level,Description) VALUES (0,'Debug');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (1,'Info');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (2,'Warning');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (3,'Error');

DELETE FROM QuizState;
INSERT INTO QuizState (Code,Description) VALUES (0,'Draft');
INSERT INTO QuizState (Code,Description) VALUES (1,'Completed');
INSERT INTO QuizState (Code,Description) VALUES (2,'Candidate');
INSERT INTO QuizState (Code,Description) VALUES (10,'Published');
INSERT INTO QuizState (Code,Description) VALUES (98,'Archived');
INSERT INTO QuizState (Code,Description) VALUES (99,'Deleted');

ALTER TABLE Person ADD Source VARCHAR(1) NULL;
ALTER TABLE Quizzes ADD State INTEGER NOT NULL DEFAULT 0;
ALTER TABLE Quizzes ADD Link VARCHAR(255) NULL;
ALTER TABLE Quizzes ADD Properties VARCHAR(65535) NULL;
ALTER TABLE Questions ADD QuestionName VARCHAR(256) NULL;
ALTER TABLE QuizResult ADD FinalPoints REAL NULL;
ALTER TABLE QuizResult ADD ReviewPoints REAL NULL;
ALTER TABLE QuizResult ADD NotAnsweredQuestsNum INTEGER NULL;
ALTER TABLE QuizResult ADD NotValuatedQuestsNum INTEGER NULL;
ALTER TABLE QuizResultQuestion ADD Points REAL NULL;
ALTER TABLE QuizResultReport ADD Charset VARCHAR(255) NULL;
ALTER TABLE QuizResultReport ADD Language VARCHAR(10) NULL;
ALTER TABLE QuizResultReport ADD Origin TEXT NULL;

ALTER TABLE Person ADD CONSTRAINT FK_PersonRole FOREIGN KEY (UserRole) REFERENCES PersonRole(ID); 
ALTER TABLE Quizzes ADD CONSTRAINT FK_QuizState FOREIGN KEY (State) REFERENCES QuizState(Code);
ALTER TABLE Categories ADD CONSTRAINT FK_CategoryQuiz FOREIGN KEY (QZ_ID) REFERENCES Quizzes(QZ_ID) ON DELETE CASCADE ON UPDATE CASCADE; 
ALTER TABLE CustomQuestionTypes ADD CONSTRAINT FK_CustomQuestionType_Quiz FOREIGN KEY (QZ_ID) REFERENCES Quizzes(QZ_ID) ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE QuizResultReport ADD CONSTRAINT FK_Report_QuizResult FOREIGN KEY (ID) REFERENCES QuizResult(ID) ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE QuizResult ALTER COLUMN UserName TYPE VARCHAR(255);
ALTER TABLE QuizResult ALTER COLUMN UserName SET NOT NULL;
ALTER TABLE QuizResult ALTER COLUMN UserLogin TYPE VARCHAR(255) ;
ALTER TABLE QuizResult ALTER COLUMN UserLogin SET NOT NULL;
ALTER TABLE QuizResult ALTER COLUMN QuizName TYPE VARCHAR(255);
ALTER TABLE QuizResult ALTER COLUMN QuizName SET NOT NULL;
ALTER TABLE QuizResult ALTER COLUMN QuizTitle TYPE VARCHAR(1024);
ALTER TABLE QuizResult ALTER COLUMN QuizTitle SET NOT NULL;
ALTER TABLE QuizResult ADD ReviewMark REAL NULL;
ALTER TABLE QuizResult ADD ReviewDate TIMESTAMP NULL;
ALTER TABLE QuizResult ADD LowestMark INTEGER NOT NULL DEFAULT 0;
ALTER TABLE QuizResult ADD RoundMark REAL NOT NULL DEFAULT 1;
ALTER TABLE QuizResult ADD IsDuplicated BOOLEAN NULL;
ALTER TABLE QuizResultQuestion ADD Feedback VARCHAR(65535) NULL;
ALTER TABLE QuizResultQuestion ADD IsCancelled BOOLEAN NULL;
ALTER TABLE QuizResultAnswer ALTER COLUMN AdditionalText TYPE VARCHAR(65535);

-- ALTER TABLE QuizResultReport ADD CONSTRAINT FK_QuizResult FOREIGN KEY (ID) REFERENCES QuizResult(ID);
-- ALTER TABLE QuizResult ADD CONSTRAINT FK_Quiz FOREIGN KEY (QuizID) REFERENCES Quizzes(QZ_ID);

ALTER TABLE Settings ADD ServerName VARCHAR(50) NULL;
ALTER TABLE Settings ADD ServerDescription VARCHAR(255) NULL;
ALTER TABLE Settings ADD ServerVersion VARCHAR(50) NULL;
ALTER TABLE Settings ADD Environment VARCHAR(50) NULL;
ALTER TABLE Settings ADD PortalBaseUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD PhpMyAdminUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD OwnerUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD FooterText VARCHAR(255) NULL;

CREATE UNIQUE INDEX indexUserIdentity ON Person (UserIdentity);

-- password : admin
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Administrator' AS PersonName,'admin' AS UserIdentity,'21232f297a57a5a743894a0e4a801fc3' AS UserPassword,1 AS UserRole,1 AS IsEnabled) AS tmp WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'admin');
-- password : anonymous
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Anonymous' AS PersonName, 'anonymous@anonymous.nowhere' AS UserIdentity,'294de3557d9d00b3d2d8a1e6aab028cf' AS UserPassword,2 AS UserRole,1 AS IsEnabled) AS tmp WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'anonymous@anonymous.nowhere');

-- QuizSession
DROP TABLE QuizSession;
CREATE TABLE QuizSession (
  PersonID INTEGER NOT NULL,
  SessionID INTEGER NOT NULL SERIAL,
  DateCreated TIMESTAMP NOT NULL,
  IpLogin VARCHAR(46) NULL,
  SessionData TEXT NULL,
  DateLastUpdate TIMESTAMP NULL,
  QuizID INTEGER NULL,
  IpLogout VARCHAR(46) NULL,
  UserAgent VARCHAR(255) NULL,
  IsRecoverable BOOLEAN NULL,
  PRIMARY KEY (SessionID),
  CONSTRAINT FK_QuizSessionPerson
    FOREIGN KEY (PersonID)
    REFERENCES Person (ID),
  CONSTRAINT FK_QuizSessionQuizResult
    FOREIGN KEY (QuizID)
    REFERENCES QuizResult (ID));

UPDATE Settings SET DbVersion = 2, DateModified=NOW(), ServerName='Node.js server', ServerDescription='Application for manage quizzes.',ServerVersion='4.1.0',Environment='DEV',PortalBaseUrl='http://localhost:3313/',OwnerUrl='https://www.quizfaber.com',FooterText='QuizFaber';